// edits by NDM to fit into smaller screens and limiting how many Serial.list() items are tried

String VERSION = "nuScope v1-3 2011-01-19";

int USBRATE = 20000;     // 100,000 max.  Bytes per second; this is conveyed to the PIC to control USB rate to avoid crashing the Mac/PC serial driver :(
int FRAMERATE = 30;     // this sets the redraw rate on the Mac/PC.  Normally 30

// screen locations and dimensions

int NCHANNELS = 8;    // two analog and the rest (six) digital

int GRAPHwide = 512, GRAPHleft = 125;      // size(850, 800) is supposed to be hardcoded in setup().  All other locations are here
int GRAPHtop[] = { 20,  280, 520, 548, 576, 604, 632, 660  }; // originally { 20,  300, 560, 592, 624, 656, 688, 720  }
int GRAPHhigh[] = { 256,  256,  32,  32,  32,  32,  32,  32 };    // this is the allocated height, but the graphs are not scaled to this.
int CHANNELLISTleft=30;
int CHANNELLISTtop[] = { 20, 300, 560, 592, 624, 656, 688, 720 };
int TIMEBASEleft=865, TIMEBASEtop=30; // originally TIMEBASEleft=690, TIMEBASEtop=170
int SWEEPMODEleft=690, SWEEPMODEtop=30;  
int TRIGGERMODEleft=690, TRIGGERMODEtop=100;  
int COUNTERleft=690, COUNTERtop=170;  // originally COUNTERleft=690, COUNTERtop=570
int PWMleft=690, PWMtop=290;  // originally PWMleft=690, PWMtop=690
int DCDCleft=690, DCDCtop=225;  // originally DCDCleft=690, DCDCtop=625
int FILENAMEleft=400, FILENAMEtop=0;
int HOTDOTleft=680, HOTDOTtop=40;  
   
// GUI labels

String SensitivityLabels[] = { "1v/div", "500mV/div", "200mV/div", "100mV/div", "50mV/div", "20mV/div", "10mV/div" }; 
float VoltsPerDivision[] = { 1.0, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01 };

String GraphLabels[] = { "Analog A", "Analog B", "Digital 1", "Digital 2", "Digital 3", "Digital 4", "output 5", "output 6" };
String TriggerModeLabels[] = { "Free run [R]", "Trigger/rising [U]", "Trigger/falling [D]" };
String SweepModeLabels[] = { "Continuous [C]", "Single sweep [S]", "Freeze [F]" };    
String Pwm5labels[] = { "Low", "10Hz sqr", "1KHz sqr", "100KHz" } ;
String Pwm6labels[] = { "Low [0]", "High [1]", "390Hz ramp", "390Hz sine" } ;

String TimeBaseLabels[] = { "1sec/div", "500mS/div", "200mS/div", "100mS/div", "50mS/div", "20mS/div", 
                            "10mS/div", "5mS/div", "2mS/div", "1mS/div", "500uS/div", "200uS/div",   
                            "100uS/div", "50uS/div", "25uS/div", "10uS/div", "5uS/div", "2.5uS/div", "1uS/div" };
float SecondsPerDivision[] = { 1.0, 0.500, 0.200, 0.100, 0.050, 0.020, 
                              0.010, 0.005, 0.002, 0.001, 0.000500, 0.000200, 
                              0.000100, 0.000050, 0.000025, 0.000010, 0.000005, 0.000001 }; 
  
// which radiobuttons to show as clickable

boolean ShowChannel[] = { true, true, true, true, true, true, true, true };  
boolean ShowSensitivity[][] = {  { true, true, true, true, true, true, true } , { true, true, true, true, true, true, true } };
boolean ShowTriggerMode[] = { true, true, true } ;
boolean ShowSweepMode[] = { true, true, true } ;
boolean ShowTimeBase[] = { true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true } ;    
boolean ChannelActive[] = { true, false, true, false, false, false, false, false } ;      // which channels are active.  First two are analog. last two are digital outpits, not inputs
boolean ShowPwm5[] = { true, true, true, true };
boolean ShowPwm6[] = { true, true, true, true };
boolean DcdcStatus = true;

// initial values for the radiobutton selected in each set

int Sensitivity[] = {   0, 0  };    
int SweepMode = 0;
int TriggerMode = 0; 
int TriggerChannel = 0;
int TimeBase = 0;
int Pwm5Mode = 0;
int Pwm6Mode = 0;
int TriggerLevel = 600;
int PixelTriggerLevel = 100;

int PicChannels, PicVersion, TIMERMASK=0x1F, TimerIndex=0, PicDcdcCurrent=0, PicDcdcStatus=0, CycleDcdcStatus=0;
int PicMilliSeconds = 0, LastPicMilliSeconds = 0;
int PicDigitalCounter = 0, LastPicDigitalCounter = 0;
int LastMillis=0;
float hertz=0;

// for USB serial communication

int PortsShown = 0;    // there is a list of ports on the screen, this is how many. 
int DelayRequested = 0;
int RedrawGui = 0;
int ShutDown = 0;
int ChangeDcdcStatus = 0;
boolean NewDcdcStatus = false;
int DataReceived = 0;

int TraceSerial = 0;    // which trace is being assembled by serial()
int TraceDraw =0;   // which trace is to be displayed by draw()

import processing.serial.*;  
Serial usbPort;  
byte[] UsbBuffer = new byte[10000];  // usb stream from the PIC  [WAY TOO BIG]
int UsbBufferIndex = 0;    // next place to write into UsbBuffer
boolean USBconnected = false;    // successful connection is established
int LastnCharTotal=0, nCharTotal=0, kBps=5;

// Traces

int[][][] Trace = new int[8][3][1000];      // first index is 8 consecutive traces.  second index [0] and [1] are analog A & B, [2] is up to 8 bits of digital
int TRACEMASK = 0x01FF;    // trace width on screen is 512 datapoints & pixels
int TraceIndex = 0;

// for screensnap (png) and data export (txt)

String filename, datestring, timestring;
int filenamefade=0;
String[] exporttext = new String[512]; 

int PAGEwide = 850, PAGEhigh = 800;    // these are also hard-coded into the "size()" command below

/* ******************************************************************************************************* */

void setup() {
  int i;
  String item;
  
  size(950, 700, P2D);    // P2D is faster than other modes
  textMode(SCREEN);    // nd it requires SCREEN mode to look decent
  background(255);
  textAlign(LEFT, CENTER);
  textSize(10);
  
  if (USBRATE < 4000) USBRATE = 4000;          // these are limits on what can be transmitted to PIC
  if (USBRATE > 63 * 4000) USBRATE = 63 * 4000;  
  
  frameRate(FRAMERATE);

  fill(255);      // clear graph area
  stroke(255);
  for (i=0; i<NCHANNELS; i++) rect(GRAPHleft, GRAPHtop[i], GRAPHwide, GRAPHhigh[i]);

  fill(0);
  stroke(0);
  text(VERSION + " @ USB " + USBRATE + " KB/sec", 150, 25, 300, 25);
  text("Click on Scope's FTDI USB serial com port, below:", 150, 50, 300, 25);     
  text("    No? Close & reopen this ap before trying a different port", 150, 75, 300, 25); 
  //println(Serial.list().length);
  for (i=0; i<Serial.list().length; i++) { // originally i<20
    try {
      fill(220, 255, 255);
      stroke(255);
      item = Serial.list()[i];      // this will throw an exception when i gets too big
      rect(150, 105+25*i, 200, 20);
      fill(0);
      text(item, 160, 100+25*i, 200, 25);
      PortsShown++;
    } 
    catch (Exception e) {
    }
  }

}

/* ******************************************************************************************************* */
// draw all the controls and axes.  Look for clicks on the radiobuttons and checkboxes and set the corresponsing variables

void drawGUI(int xClick, int yClick) {
  int i;

  TimeBase = radioButtonSet(TimeBase, xClick, yClick, 19, ShowTimeBase,  TimeBaseLabels, TIMEBASEleft, TIMEBASEtop, 100, 20) ;          // first, because a lot of mode availability depends on this
  
  NewDcdcStatus = checkBox(DcdcStatus, xClick, yClick, 0, true, "Power supply", DCDCleft, DCDCtop, 120, 20);
  
  if (DcdcStatus != NewDcdcStatus) {
    ChangeDcdcStatus++;
  }
  
  for (i=0; i<2; i++) {    //checkBox(boolean state, int xClick, int yClick, int IGNORE, boolean show, String label, int xCorner, int yCorner, int xWidth, int yInterval) {
    ChannelActive[i] = checkBox(ChannelActive[i], xClick, yClick, 0, ShowChannel[i], GraphLabels[i], CHANNELLISTleft, GRAPHtop[i]+30, 90, 20);
    
    ShowSensitivity[i][0] = ShowSensitivity[i][1] = ShowSensitivity[i][2] = ShowSensitivity[i][3] = ShowSensitivity[i][4] = ShowSensitivity[i][5] = ShowSensitivity[i][6] = ChannelActive[i] ;
    
    Sensitivity[i] = radioButtonSet(Sensitivity[i], xClick, yClick, 7, ShowSensitivity[i],  SensitivityLabels, CHANNELLISTleft, GRAPHtop[i]+60, 80, 20) ;
    if (ChannelActive[i]) draw2Axes(GRAPHleft, GRAPHtop[i], GRAPHwide, GRAPHhigh[i], 50, 25);
  }

  for (i=2; i<NCHANNELS; i++) {
    ChannelActive[i] = checkBox(ChannelActive[i], xClick, yClick, 0, ShowChannel[i], GraphLabels[i], CHANNELLISTleft, GRAPHtop[i]+15, 100, 20);  
    if (ChannelActive[i]) draw1Axis(GRAPHleft, GRAPHtop[i], GRAPHwide, GRAPHhigh[i], 50, 120);
  }
    
  SweepMode = radioButtonSet(SweepMode, xClick, yClick, 3, ShowSweepMode,  SweepModeLabels, SWEEPMODEleft, SWEEPMODEtop, 120, 20) ;
  
  TriggerMode = radioButtonSet(TriggerMode, xClick, yClick, 3, ShowTriggerMode,  TriggerModeLabels, TRIGGERMODEleft, TRIGGERMODEtop, 120, 20) ;
  
  
  ShowPwm6[2] = ShowPwm6[3] = (TimeBase < 11) ;
  if (TimeBase >= 11 && Pwm6Mode > 1) Pwm6Mode = 0;
  Pwm5Mode = radioButtonSet(Pwm5Mode, xClick, yClick, 4, ShowPwm5,  Pwm5labels, PWMleft, PWMtop, 100, 15) ;
  Pwm6Mode = radioButtonSet(Pwm6Mode, xClick, yClick, 4, ShowPwm6,  Pwm6labels, PWMleft+70, PWMtop, 100, 15) ;
    


// a click in the graphs is used to set the trigger level

  if (TriggerMode != 0 && xClick > GRAPHleft && xClick < GRAPHleft + GRAPHwide + 25) {
    
    for (i=0; i<2; i++) {    //analog channels
      if (ChannelActive[i] &&  (yClick > GRAPHtop[i])  && ( yClick < GRAPHtop[i]+GRAPHhigh[i] ) ) {          // triggerlevel is within which trace?
        TriggerChannel = i;
        TriggerLevel = 1024 - 4 * (yClick - GRAPHtop[i]);
        PixelTriggerLevel = yClick;
      }
    }
    
    for (i=2; i<NCHANNELS; i++) {    // digital channels
      if (ChannelActive[i] && (yClick > GRAPHtop[i] +GRAPHhigh[i]/4)  && ( yClick < GRAPHtop[i]+GRAPHhigh[i]+GRAPHhigh[i]/4 ) ) {          // triggerlevel is within which trace?
        TriggerChannel = i;
        PixelTriggerLevel = GRAPHtop[i]+GRAPHhigh[i]- 5;
      }
    }
  }
}   


/* ******************************************************************************************************* */
 // Read zero-terminated packets and put data into the Trace array.  But don't draw anything on the screen; that's done by draw().  
 
void serialEvent(Serial p) {     
  int i, nChar, j, packettype = 0;
 
  nChar = usbPort.readBytesUntil(0, UsbBuffer);
  nCharTotal += nChar;

  if (nChar < 2) return;
  i = 0;
  packettype = UsbBuffer[i++];
  
  
  if (packettype == 0x10) {    // header packet
    PicVersion = UsbBuffer[i++] & 0x03F;
    PicDcdcCurrent = ((UsbBuffer[i++] & 0x03F) << 6) | (UsbBuffer[i++] & 0x03F);
    PicMilliSeconds = ((UsbBuffer[i++] & 0x03F) << 24) | ((UsbBuffer[i++] & 0x03F) << 18) | ((UsbBuffer[i++] & 0x03F) << 12) | ((UsbBuffer[i++] & 0x03F) << 6) | ((UsbBuffer[i++] & 0x03F));
    PicDigitalCounter = ((UsbBuffer[i++] & 0x03F) << 24) | ((UsbBuffer[i++] & 0x03F) << 18) | ((UsbBuffer[i++] & 0x03F) << 12) | ((UsbBuffer[i++] & 0x03F) << 6) | ((UsbBuffer[i++] & 0x03F));   
    
    
    if (abs(PicMilliSeconds - LastPicMilliSeconds) >= 500) {
      hertz =  1000.0 * (PicDigitalCounter - LastPicDigitalCounter) / (PicMilliSeconds - LastPicMilliSeconds);      
      LastPicDigitalCounter = PicDigitalCounter;
      LastPicMilliSeconds = PicMilliSeconds;
    } 
    
    if (((PicDcdcCurrent & 0x01) == 0) != DcdcStatus) {    // low bit tells if DCDC is disabled.   
      DcdcStatus = !DcdcStatus;
      RedrawGui++;   // low bit shows its disabled
    }
  }
  
  if (packettype > 0 && packettype < 16) {    // data packet with up to 4 words per datapoint specified
    DataReceived = 1;    // will show it as "running" on screen for a few frames
    TraceIndex = ((UsbBuffer[i++] & 0x03F) << 6) | (UsbBuffer[i++] & 0x03F);    // where on the time axis to put this data

     for (; i<nChar-1; ) {     
       if ((packettype & 0x01) != 0) Trace[TraceSerial][0][TraceIndex & TRACEMASK] = ((UsbBuffer[i++] & 0x03F) << 6) | (UsbBuffer[i++] & 0x03F);
       if ((packettype & 0x02) != 0) Trace[TraceSerial][1][TraceIndex & TRACEMASK] = ((UsbBuffer[i++] & 0x03F) << 6) | (UsbBuffer[i++] & 0x03F);
       if ((packettype & 0x04) != 0) Trace[TraceSerial][2][TraceIndex & TRACEMASK] = ((UsbBuffer[i++] & 0x03F) << 6) | (UsbBuffer[i++] & 0x03F);
       
       if (!DcdcStatus) Trace[TraceSerial][1][TraceIndex & TRACEMASK] = Trace[TraceSerial][0][TraceIndex & TRACEMASK] = 1024;    // show bad data if DC is off
       
       TraceIndex++;

// the Sweep Mode functionality is implemented here

       if ((TraceIndex & TRACEMASK) == 0) {    // a trace has been completed.  IS THIS A ROBUST WAY TO KNOW THAT?
       
       
         if (SweepMode == 0 || SweepMode == 1) {    // continuous update mode (or single sweep mode)
           TraceDraw = TraceSerial;    // start drawing from the completed trace
           if (TimeBase > 5) TraceSerial = ((TraceSerial+1) & 7);    // only at high speeds, start writing into a new trace buffer
         }
         
         if (SweepMode == 2) {    // in freeze mode, draw buffer is not advanced, but new data goes to new buffers
             TraceSerial = ((TraceSerial+1) & 7);    // only at high speeds, start writing into a new trace buffer
             if (TraceSerial == TraceDraw) TraceSerial = ((TraceSerial+1) & 7);    // protect the draw buffer from being overwritten
         }
         
         if (SweepMode == 1) {
           SweepMode = 2;      // freeze after a sweep has finished
           RedrawGui = 1;
         }
       }
     }

    if ( (TimeBase < 5) && (TraceIndex & TRACEMASK) > 0  && (TraceIndex & TRACEMASK) < 508 ) {      // at low sweep rates, clear a few pixels ahead to show an overwrite is going on
      for (i= (TraceIndex & TRACEMASK); i < (TraceIndex & TRACEMASK) + 10; i++) Trace[TraceSerial][0][i] = Trace[TraceSerial][1][i] = -1;
    }  
  }
}
 
/* ******************************************************************************************************* */
// this runs FRAMERATE times per second, typically 30 

void draw() {      
  int i, nChar, deltatime, channel;


  if (DelayRequested > 0) {      // pause to display startup messages
    DelayRequested--;
    return;
  }
  
  if (RedrawGui > 0) {
      drawGUI(0, 0);
      RedrawGui = 0;
  }
  
  if (USBconnected) {

    fill(255);      // clear graph area
    stroke(255);
  
    for (i=0; i<NCHANNELS; i++) rect(GRAPHleft, GRAPHtop[i], GRAPHwide, GRAPHhigh[i]);
    
    rect(GRAPHleft+GRAPHwide+10, GRAPHtop[0], 15, PAGEhigh-GRAPHtop[0]);
  
    for (i=0; i<2; i++) if (ChannelActive[i]) draw2Axes(GRAPHleft, GRAPHtop[i], GRAPHwide, GRAPHhigh[i], 50, 25);
  
    for (i=2; i<NCHANNELS; i++) if (ChannelActive[i]) draw1Axis(GRAPHleft, GRAPHtop[i], GRAPHwide, GRAPHhigh[i], 50, 120); 
    

  
    strokeWeight(1);
    
    if (ChannelActive[0]) {        // turn off A at 5uS/div and faster
         for (i=0; i<512; i++) {
           stroke(0,0,255);    // blue
           if (TimeBase > 15) stroke(128);    // grey
           if (Trace[TraceDraw][0][i] >= 1024) stroke(255,0,0);    // red
           point(i + GRAPHleft, GRAPHtop[0]+GRAPHhigh[0] - Trace[TraceDraw][0][i]/4);
         }
    }

    if (ChannelActive[1] /*&& TimeBase < 15*/) {      // turn off B at 10uS/div and faster
         for (i=0; i<512; i++) {
           stroke(0,0,255);    // blue
           if (TimeBase > 14) stroke(128);    // grey
           if (Trace[TraceDraw][1][i] >= 1024) stroke(255,0,0);    // red
           point(i + GRAPHleft, GRAPHtop[1]+GRAPHhigh[1] - Trace[TraceDraw][1][i]/4);
         }    
    }    


      for (channel=2; channel<NCHANNELS; channel++) {
         if (ChannelActive[channel]) {
           if (TimeBase <= 16) drawDigitalGraph(channel, 1, 1);          // second arg is expansion factor horizontally, third arg is the pixel width
           if (TimeBase == 17) drawDigitalGraph(channel, 2, 2);
           if (TimeBase == 18) drawDigitalGraph(channel, 5, 3);
         }
      }
           

     
    if (TriggerMode != 0) {
        if(TriggerChannel < 2) {        // show the triggerlevel on the trace 
           strokeWeight(1);    // draw new stroke
           stroke(255, 160, 160);    // pink
           line(GRAPHleft+5, PixelTriggerLevel, GRAPHleft+GRAPHwide-5, PixelTriggerLevel);
        }
        strokeWeight(3);    // draw new stroke
        stroke(255, 100, 100);    // pink
        line(GRAPHleft+GRAPHwide+10, PixelTriggerLevel, GRAPHleft+GRAPHwide+25, PixelTriggerLevel);
    }
    
    
    strokeWeight(1);    // write the top line
    fill(255);          // version ID for nuScope processing code & PIC firmware code
    noStroke();
    rect(0, 0, PAGEwide, 15);
    fill(100);
    textAlign(LEFT, CENTER);
    
    if (millis() > LastMillis + 500) {
         kBps = (nCharTotal - LastnCharTotal) / (millis() - LastMillis);
         LastnCharTotal = nCharTotal;
         LastMillis = millis();
    }
    text(" " + VERSION + " + firmware v." + nf(PicVersion, 0) + " Time=" + nf((PicMilliSeconds/1000.0), 7, 1) + " kBps=" + nf(kBps, 0), 0, 0, 350, 15);

     fill(255);      // filename when a .txt or .png file has been written
     
     rect( FILENAMEleft, FILENAMEtop, 200, 15);
     
    if (filenamefade > 0) {          // draw fading filename last written. 
          
          if (filenamefade < FRAMERATE/2) {    // red for 1/2 second
               fill(255,0,0);    
               text("Writing " + filename, FILENAMEleft, FILENAMEtop, 200, 15);
          }
          else {
               fill(128);
               text("Wrote " + filename, FILENAMEleft, FILENAMEtop, 200, 15);
          }
          if (filenamefade++ > 10*FRAMERATE) filenamefade=0;    // then grey for 10 seconds
     }
     else {
          fill(128);
          text("[Q] Quit, [P] .png screen snap, [X] .txt data export", FILENAMEleft, FILENAMEtop, 300, 15);
     }


  
    fill(255);      // counter and frequenecy counter from PCB hole D1
    rect(COUNTERleft, COUNTERtop, 150, 50);
    fill(0);
    textAlign(LEFT, CENTER);
    text("D1 count & frequency", COUNTERleft, COUNTERtop, 150, 15);    
    textAlign(RIGHT, CENTER);
    text(nf(PicDigitalCounter, 0) , COUNTERleft, COUNTERtop+15, 100, 15);   
    text(nf(hertz, 6, 1), COUNTERleft, COUNTERtop+30, 100, 15);

    
    fill(255);      // PWM choices
    rect(PWMleft, PWMtop-15, 150, 15);
    fill(0);
    textAlign(LEFT, CENTER);
    text("Output O5      Output O6", PWMleft, PWMtop-15, 150, 15);     
    
    stroke(1);          // bar graph for DCDC current
    fill(255);
    rect(DCDCleft+25, DCDCtop+24, 60, 10);
    fill(200);
    rect(DCDCleft+25, DCDCtop+24, min(PicDcdcCurrent/20, 60), 10);

    
    SendPacketToPic();          // send instructions to the PIC
    
    if (DataReceived > 0) {    // hot dot - grey or red
      fill(255, 180, 180);
      DataReceived--;
    }
    else {
      fill(180);
    }
    stroke(1);
    ellipse(HOTDOTleft, HOTDOTtop, 10, 10);
  
    
    
  }   
  
  if (ShutDown > 0) {
 //     usbPort.stop();
      fill(230);
      stroke(0);
      rect(0, 0, PAGEwide, PAGEhigh);
      fill(0);
      textAlign(CENTER, CENTER);
      text("Scope paused; thanks for using Q.  OK to close this window.", 0, PAGEhigh/2, PAGEwide, 20);
//      text("Did you know about P (export window image to .png file) and X (export data to .txt file)", 0, PAGEhigh/2+40, PAGEwide, 20);
      exit();
  }
  

  return;
}

/* ******************************************************************************************************* */
// draw trace 

void drawDigitalGraph( int channel, int expansion, int dotpixels) {
     
     int i, baseline, high, low;
     
     baseline = GRAPHtop[channel]+GRAPHhigh[channel];
     
      stroke(128,128,255);
      fill(128,128,255);
      
      for (i=0; i<512/expansion; i++) {
          high = (Trace[TraceDraw][2][i] >> (channel - 2)) & 1;          // 1 if signal was ever high in this sample
          low = (Trace[TraceDraw][2][i] >> (channel + 4)) & 1;           // 0 if signal was ever low in this sample
     
          if (high == 0 && low == 1) continue;          // shouldn't happen: neither high nor low seen
          if (low == 0) rect(GRAPHleft+expansion*i, baseline - 11, dotpixels, 5);
          if (high == 1) rect(GRAPHleft+expansion*i, baseline - 5, dotpixels, 5);
     }
}
     

/* ******************************************************************************************************* */
// draw trace grids

void draw2Axes(int xleft, int ytop, int xwidth, int yheight, int xdivpix, int ydivpix) {
  int i, divs, where;

  pushMatrix();

  stroke(200);
  strokeWeight(1);

  translate(xleft, ytop);

  line(-1, -1, xwidth+2, -1);      // horizontals
  line(-1, yheight+1, xwidth+2, yheight+1);
  for (i=0; i<=xwidth; i+=xdivpix) {
    line(i, -6, i, -1);
    line(i, yheight+1, i, yheight+6);
  }

  line(-1, -1, -1, yheight+2);    // verticals
  line(xwidth+1, -1, xwidth+1, yheight+2);
  if (ydivpix > 0) {
    divs = yheight/ydivpix;
    for (i=-divs/2; i<=divs/2; i++) {
      where = yheight/2 + i * ydivpix;
      line(-6, where, -1, where);
      line(xwidth+1, where, xwidth+6, where);
    }

    line(-1, yheight/2, xwidth+2, yheight/2);    // volts = 0 line
  }
  popMatrix();    

  return;
}

/* ******************************************************************************************************* */
// draw time axis only

void draw1Axis(int xleft, int ytop, int xwidth, int yheight, int xdivpix, int ydivpix) {
  int i, divs, where;

  stroke(200);
  strokeWeight(1);
  pushMatrix();
  translate(xleft, ytop);

  line(0, yheight, xwidth, yheight);
  for (i=0; i<=xwidth; i+=xdivpix) {

    line(i, yheight, i, yheight+5);
  }
  popMatrix();
}


/* ******************************************************************************************************* */
// look for buttons or radio boxes clicked, or (if in select-a-port mode) look for a port name clicked on and open it

void mouseClicked()      // click and released  
{
  int i;

  if (USBconnected) {   
    drawGUI( mouseX, mouseY );      // draw the gui and look for clicked buttons
  }

  if (PortsShown > 0) {                  // if we are in select-a-port mode, open a port
    if ( mouseX > 150 && mouseX < 350) {
      for (i=0; i<PortsShown; i++) {
        if ( mouseY > 105+25*i && mouseY < 125+25*i ) {    


          try {
            usbPort = new Serial(this, Serial.list()[i], 9600);    // baud ignored, it's 1Mbyte/sec
            println(usbPort);
            usbPort.bufferUntil(0);        // call serialevent at the end of each 0-terminated packet receieved  
            usbPort.clear();    // might this help not crash?      PROBLABY NOT   
            fill(255);
            rect(150, 75+25*PortsShown, 200, 25);
            fill(0);
            text("CONNECTED OK", 150, 75+25*PortsShown, 200, 25);
            text("Please use Q to quit.  X for data export (.txt file)  P for image (.png file)", 150, 100+25*PortsShown, 400, 25);
           
            DelayRequested = FRAMERATE;    // 1 second delay to show this message
            PortsShown = 0;
            USBconnected = true;
          } 
          catch (Exception e) {
            exit();        // unfortunately USB exception is not caught.   It just hangs.
          }
        }
      }
    }
    drawGUI(0, 0);
  }
}


/* ******************************************************************************************************* */
// ping the PIC32 to tell it we are here eating USB stream, and encoding the scope control parameters

void SendPacketToPic() {      // send a 20-byte USB ping to the PIC (18 bytes of content)
  int channelcode = 0;

  if (USBconnected) {

    channelcode = 0;          // encode which channels should be digitized by the PIC
    if (ChannelActive[0]) channelcode += 1;
    if (ChannelActive[1]) channelcode += 2;
    if (ChannelActive[2] | ChannelActive[3] | ChannelActive[4] | ChannelActive[5] | ChannelActive[6] | ChannelActive[7]) channelcode +=4;
  
    usbPort.write(0x01);    // byte 0: delimit packet by a 1 and a 2.  The content bytes have 01 000 000 added (0x40)
    
         usbPort.write(0x40 + Sensitivity[0]);    // 1
         usbPort.write(0x40 + Sensitivity[1]);    // 2
         usbPort.write(0x40 + SweepMode);     // 3
         usbPort.write(0x40 + TimeBase);    // 4       
         usbPort.write(0x40 + TriggerMode);  // 5
         usbPort.write(0x40 + TriggerChannel);     // 6
         usbPort.write(0x40 + ((TriggerLevel>>5) & 0x1F));    // 7: high 5 bits of the trigger level
         usbPort.write(0x40 + ((TriggerLevel) & 0x1F));    // 8: low 5 bits of the trigger level
         usbPort.write(0x40 + channelcode);    // 9: which analog channels we want the PIC to look at     
         usbPort.write(0x40 + Pwm5Mode);  // 10
         usbPort.write(0x40 + (int(USBRATE/4000) & 0x03F));    // 11: range 4KB/s to 252KB/s
         usbPort.write(0x40 + (ChangeDcdcStatus & 0x01));     // 12
         usbPort.write(0x40 + Pwm6Mode);    // 13  Sine wave cycles
         usbPort.write(0x40 + 0);     // 14     
         usbPort.write(0x40 + 0);    // 15
         usbPort.write(0x40 + 0);     // 16
         usbPort.write(0x40 + 0);     // 17
         usbPort.write(0x40 + 0);    // 18
    
    usbPort.write(0x02);     // byte 19 (bytes are numbered 0 to 19)
  }
  return;
}

/* ******************************************************************************************************* */
// just Q for quit

void keyPressed() {
  int mildred, i;

     RedrawGui = 1;          // may need to redraw to show any modes that have been changed
     
     if ( (key & 0x7F) == ('0' & 0x7F) ) Pwm6Mode = 0;  
     else if ( (key & 0x7F) == ('1' & 0x7F) ) Pwm6Mode = 1;  

     else if ( (key & 0x1F) == ('c' & 0x1F) ) SweepMode = 0;    // 0x1F so small, capital, and ctrl/ all work
     else if ( (key & 0x1F) == ('s' & 0x1F) ) SweepMode = 1;
     else if ( (key & 0x1F) == ('f' & 0x1F) ) SweepMode = 2;
     else if ( (key & 0x1F) == ('r' & 0x1F) ) TriggerMode = 0;
     else if ( (key & 0x1F) == ('u' & 0x1F) ) TriggerMode = 1;
     else if ( (key & 0x1F) == ('d' & 0x1F) ) TriggerMode = 2;      
    
     else if ( (key & 0x1F) == ('q' & 0x1F) ) {  // any variation on Q       

         ShutDown = 1;          // signal to draw() to call exit()
         TimeBase = 0;
         TriggerMode = 0;
         SweepMode = 2;  // stop the pic
         USBRATE = 0;
         SendPacketToPic();                  
     
         mildred = millis() + 100;
         while (millis() < mildred) ;    // wait 100 msec for PIC to shut up     
     }
     
     else if ( (key & 0x1F) == ('p' & 0x1F) ) {                // PRINT: capture screenshot to png file
          datestring =  String.valueOf(10000*year() + 100*month() + day());
          timestring = String.valueOf(100000*hour() + 1000*minute() + 10*second() + int(millis()/100));
          if (timestring.length() == 6) timestring = "0" + timestring;    // 24 hour time - prepend zero 
          filename = "nuScope_" + datestring + "_" + timestring + ".png";
          filenamefade=1;
          save(filename);
     }

     else if ( (key & 0x1F) == ('x' & 0x1F) ) {                // EXPORT: export txt file
          datestring =  String.valueOf(10000*year() + 100*month() + day());
          timestring = String.valueOf(100000*hour() + 1000*minute() + 10*second() + int(millis()/100));
          if (timestring.length() == 6) timestring = "0" + timestring;    // 24 hour time - prepend zero 
          filename = "nuScope_" + datestring + "_" + timestring + ".txt";
          filenamefade=1;
          for (i=0; i<512; i++) {          // !!! create tab delimited file for export
               exporttext[i] = String.valueOf(i * SecondsPerDivision[TimeBase] / 50.0) 
               + "\t" +  String.valueOf((Trace[TraceDraw][0][i]-512) * VoltsPerDivision[Sensitivity[0]] / 100.0) 
               + "\t" +  String.valueOf((Trace[TraceDraw][1][i]-512) * VoltsPerDivision[Sensitivity[1]] / 100.0) 
               + "\t" +  String.valueOf((Trace[TraceDraw][2][i] & 0x01)) + "\t" +  String.valueOf((Trace[TraceDraw][2][i] & 0x02)>>1)
               + "\t" +  String.valueOf((Trace[TraceDraw][2][i] & 0x04)>>2) + "\t" +  String.valueOf((Trace[TraceDraw][2][i] & 0x08)>>3)
               + "\t" +  String.valueOf((Trace[TraceDraw][2][i] & 0x10)>>4) + "\t" +  String.valueOf((Trace[TraceDraw][2][i] & 0x20)>>5);
          }
          saveStrings(filename, exporttext);
     }

}



/* ******************************************************************************************************* */
// draw radiobuttons, while looking for ones that are being clicked

int radioButtonSet(int state, int xClick, int yClick, int nButtons, boolean show[], String labels[], int xCorner, int yCorner, int xWidth, int yInterval) {
  int i;

  textAlign(LEFT, CENTER);
  strokeWeight(1);
  stroke(255);
  fill(255);
  rect(xCorner, yCorner, xWidth, nButtons * yInterval);    // clear your space

  for (i=0; i<nButtons; i++) {      // type the labels and look for clicked buttons
    strokeWeight(1);
    stroke(255);
    if (show[i]) fill(0);
    else fill(128);
    text(labels[i], xCorner+20, yCorner + i*yInterval, xWidth-20, 20);
    if (show[i] && dist( xCorner + 10, yCorner + i*yInterval + 10, xClick, yClick) < 10) state = i;
  }

  for (i=0; i<nButtons; i++) {      // draw the buttons
    if (show[i]) {
      strokeWeight(1);
      stroke(0);
      fill(255);
      ellipse(xCorner+10, yCorner + i*yInterval + 10, 10, 10);
      if (i == state) {
        fill(0);
        ellipse(xCorner+10, yCorner + i*yInterval + 10, 5, 5);
      }
    }
  }
  return(state);    // this is the new (or the old) selected button
}


/* ******************************************************************************************************* */
// draw one checkbox, while looking to see if it is being clicked

boolean checkBox(boolean state, int xClick, int yClick, int IGNORE, boolean show, String label, int xCorner, int yCorner, int xWidth, int yInterval) {

  textAlign(LEFT, CENTER);
  strokeWeight(1);
  stroke(255);
  fill(255);
  rect(xCorner, yCorner, xWidth, yInterval);    // clear your space

  fill(0);      // type the labels and look for clicked buttons

  text(label, xCorner+20, yCorner, xWidth-20, 20);
  if (show && dist( xCorner + 10, yCorner + 10, xClick, yClick) < 12) state = !state;
  stroke(0);

  if (show) {            // this is for the checkboxes; turned off so only the labels are shown (always)
    fill(255);
    rect(xCorner+4, yCorner+4, 12, 12);
    if (state) {
      fill(0);
      strokeWeight(1);
      line(xCorner+4, yCorner+4, xCorner+16, yCorner+16);
      line(xCorner+16, yCorner+4, xCorner+4, yCorner+16);
      strokeWeight(1);
    }
  }

  return(state);    // this is the new (or the old) value of the checkbox
}      

/* ************************************** END ************************************************************ */




